@@ -1,7 +1,7 @@
-// 
+//
 
 /* This file is part of the KDE project
-   Copyright 2001, 2002, 2003 Nicolas GOUTTE <goutte@kde.org>
+   Copyright 2001, 2002, 2003, 2004 Nicolas GOUTTE <goutte@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -38,7 +38,7 @@
 #include <kgenericfactory.h>
 #include <kmessagebox.h>
 
-#include <koGlobal.h>
+#include <koPageLayout.h>
 #include <koStore.h>
 #include <koFilterChain.h>
 
@@ -57,7 +57,7 @@ public:
 protected:
     virtual void setupTranslations( void )
     {
-        KGlobal::locale()->insertCatalogue( "kwordabiwordfilter" );
+        KGlobal::locale()->insertCatalogue( "kofficefilters" );
     }
 };
 
@@ -118,7 +118,7 @@ private:
         const QXmlAttributes& attributes);
     bool StartElementFoot(StackItem* stackItem, StackItem* stackCurrent,
         const QXmlAttributes& attributes);
-    bool StartElementTable(StackItem* stackItem, StackItem* stackCurrent);
+    bool StartElementTable(StackItem* stackItem, StackItem* stackCurrent, const QXmlAttributes& attributes);
     bool StartElementCell(StackItem* stackItem, StackItem* stackCurrent,const QXmlAttributes& attributes);
 private:
     void createDocument(void);
@@ -143,8 +143,6 @@ private:
     bool m_fatalerror;                  // Did a XML parsing fatal error happened?
 };
 
-
-
 // Element <c>
 
 bool StructureParser::StartElementC(StackItem* stackItem, StackItem* stackCurrent, const QXmlAttributes& attributes)
@@ -352,7 +350,7 @@ bool StartElementP(StackItem* stackItem, StackItem* stackCurrent,
     QDomElement elementText=stackCurrent->stackElementText;
     QDomElement paragraphElementOut=mainDocument.createElement("PARAGRAPH");
     stackCurrent->m_frameset.appendChild(paragraphElementOut);
-    
+
     QDomElement textElementOut=mainDocument.createElement("TEXT");
     paragraphElementOut.appendChild(textElementOut);
     QDomElement formatsPluralElementOut=mainDocument.createElement("FORMATS");
@@ -734,7 +732,7 @@ static bool StartElementM(StackItem* stackItem, StackItem* /*stackCurrent*/,
         stackItem->elementType=ElementTypeIgnore;
         return true;
     }
-    
+
     stackItem->strTemp1=strKey;        // Key
     stackItem->strTemp2=QString::null;  // Meta data
 
@@ -755,17 +753,17 @@ bool StructureParser::EndElementM (StackItem* stackItem)
         kdError(30506) << "Wrong element type!! Aborting! (in endElementM)" << endl;
         return false;
     }
-    
+
     if (stackItem->strTemp1.isEmpty())
     {
         // Probably an internal error!
         kdError(30506) << "Key name was erased! Aborting! (in endElementM)" << endl;
         return false;
     }
-    
+
     // Just add it to the metadata map, we do not do something special with the values.
     m_metadataMap[stackItem->strTemp1]=stackItem->strTemp2;
-    
+
     return true;
 }
 
@@ -1108,13 +1106,28 @@ bool StructureParser::StartElementFoot(StackItem* stackItem, StackItem* /*stackC
 }
 
 // Element <table>
-
-bool StructureParser::StartElementTable(StackItem* stackItem, StackItem* stackCurrent)
+bool StructureParser::StartElementTable(StackItem* stackItem, StackItem* stackCurrent,
+    const QXmlAttributes& attributes)
 {
-#if 0
+#if 1
     // In KWord, inline tables are inside a paragraph.
     // In AbiWord, tables are outside any paragraph.
 
+    QStringList widthList;
+    widthList.split('/', attributes.value("table-column-props"), false);
+    const uint columns = widthList.size();
+    stackItem->m_doubleArray.detach(); // Be sure not to modify parents
+    stackItem->m_doubleArray.resize(columns+1); // All left positions but the last right one
+    stackItem->m_doubleArray[0] = 0.0;
+    QStringList::ConstIterator it;
+    uint i;
+    for ( i=0, it=widthList.begin(); i<columns; ++i, ++it )
+    {
+        kdDebug(30506) << "Column width: " << (*it) << " cooked " << ValueWithLengthUnit(*it) << endl;
+        stackItem->m_doubleArray[i+1] = ValueWithLengthUnit(*it) + stackItem->m_doubleArray[i];
+    }
+    // ### TODO: in case of automatic column widths, we have not any width given by AbiWord
+
     const uint tableNumber(++m_tableGroupNumber);
     const QString tableName(i18n("Table %1").arg(tableNumber));
 
@@ -1165,7 +1178,7 @@ bool StructureParser::StartElementTable(StackItem* stackItem, StackItem* stackCu
 bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCurrent,
     const QXmlAttributes& attributes)
 {
-#if 0
+#if 1
     if (stackCurrent->elementType!=ElementTypeTable)
     {
         kdError(30506) << "Wrong element type!! Aborting! (in StructureParser::endElementCell)" << endl;
@@ -1190,6 +1203,15 @@ bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCur
     const uint row=abiPropsMap["top-attach"].getValue().toUInt();
     const uint col=abiPropsMap["left-attach"].getValue().toUInt();
 
+    if ( col >= stackItem->m_doubleArray.size() )
+    {
+        // We do not know the right position of this column, so improvise. (### TODO)
+        // We play on the fact that QByteArray uses shallow copies by default.
+        //  (We do want that the change is known at <table> level)
+        stackItem->m_doubleArray.resize( stackItem->m_doubleArray.size() + 1, QGArray::SpeedOptim );
+        stackItem->m_doubleArray[col+1] = stackItem->m_doubleArray[col] + 72; // Try 1 inch
+    }
+
     const QString frameName(i18n("Frameset name","Table %3, row %1, column %2")
         .arg(row).arg(col).arg(stackCurrent->strTemp2)); // As the stack could be wrong, be careful and use the string as last!
 
@@ -1207,15 +1229,15 @@ bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCur
     framesetsPluralElement.appendChild(framesetElement);
 
     QDomElement frameElementOut(mainDocument.createElement("FRAME"));
-    //frameElementOut.setAttribute("left",28);
-    //frameElementOut.setAttribute("right",798);
+    frameElementOut.setAttribute( "left", stackItem->m_doubleArray[col] );
+    frameElementOut.setAttribute( "right", stackItem->m_doubleArray[col+1] );
     frameElementOut.setAttribute("top",0);
     frameElementOut.setAttribute("bottom",0);
     frameElementOut.setAttribute("runaround",1);
     frameElementOut.setAttribute("autoCreateNewFrame",0); // Very important for cell growing!
     // ### TODO: a few attributes are missing
     framesetElement.appendChild(frameElementOut);
-    
+
     stackItem->m_frameset=framesetElement;
     QDomElement nullDummy;
     stackItem->stackElementParagraph=nullDummy; // <PARAGRAPH>
@@ -1360,7 +1382,7 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
     }
     else if (name=="table") // No upper-case
     {
-        success=StartElementTable(stackItem,structureStack.current());
+        success=StartElementTable(stackItem,structureStack.current(), attributes);
     }
     else if (name=="cell") // No upper-case
     {
@@ -1539,14 +1561,22 @@ void StructureParser::createDocInfo(void)
 
     QDomElement about(mainDocument.createElement("about"));
     elementDoc.appendChild(about);
-    
+
     QDomElement abstract(mainDocument.createElement("abstract"));
     about.appendChild(abstract);
     abstract.appendChild(mainDocument.createTextNode(m_metadataMap["dc.description"]));
-    
+
     QDomElement title(mainDocument.createElement("title"));
     about.appendChild(title);
     title.appendChild(mainDocument.createTextNode(m_metadataMap["dc.title"]));
+
+    QDomElement keyword(mainDocument.createElement("keyword"));
+    about.appendChild(keyword);
+    keyword.appendChild(mainDocument.createTextNode(m_metadataMap["abiword.keywords"]));
+
+    QDomElement subject(mainDocument.createElement("subject"));
+    about.appendChild(subject);
+    subject.appendChild(mainDocument.createTextNode(m_metadataMap["dc.subject"]));    
 }
 
 bool StructureParser::endDocument(void)
@@ -1557,7 +1587,7 @@ bool StructureParser::endDocument(void)
 
     kdDebug(30506) << "###### Start Style List ######" << endl;
     StyleDataMap::ConstIterator it;
-    
+
     // At first, we put the Normal style
     it=styleDataMap.find("Normal");
     if (it!=styleDataMap.end())
@@ -1585,7 +1615,7 @@ bool StructureParser::endDocument(void)
     kdDebug(30506) << "######  End Style List  ######" << endl;
 
     createDocInfo();
-    
+
     return true;
 }
 
@@ -1633,7 +1663,7 @@ void StructureParser :: createDocument(void)
     elementDoc.setAttribute("xmlns","http://www.koffice.org/DTD/kword");
     elementDoc.setAttribute("editor","AbiWord Import Filter");
     elementDoc.setAttribute("mime","application/x-kword");
-    elementDoc.setAttribute("syntaxVersion",3);
+    elementDoc.setAttribute( "syntaxVersion", 3 );
     mainDocument.appendChild(elementDoc);
 
     QDomElement element;
@@ -1773,9 +1803,15 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QCString& from, const Q
 
     QIODevice* in = KFilterDev::deviceForFile(fileIn,strMime);
 
+    if ( !in )
+    {
+        kdError(30506) << "Cannot create device for uncompressing! Aborting!" << endl;
+        return KoFilter::FileNotFound; // ### TODO: better error?
+    }
+
     if (!in->open(IO_ReadOnly))
     {
-        kdError(30506) << "Cannot open file! Aborting!" << endl;
+        kdError(30506) << "Cannot open file for uncompressing! Aborting!" << endl;
         delete in;
         return KoFilter::FileNotFound;
     }
@@ -1800,7 +1836,7 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QCString& from, const Q
 
     QCString strOut;
     KoStoreDevice* out;
-    
+
     kdDebug(30506) << "Creating documentinfo.xml" << endl;
     out=m_chain->storageFile( "documentinfo.xml", KoStore::Write );
     if(!out)
@@ -1814,7 +1850,7 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QCString& from, const Q
     strOut=handler.getDocInfo().toCString(); // UTF-8
     // WARNING: we cannot use KoStore::write(const QByteArray&) because it writes an extra NULL character at the end.
     out->writeBlock(strOut,strOut.length());
-    
+
     kdDebug(30506) << "Creating maindoc.xml" << endl;
     out=m_chain->storageFile( "root", KoStore::Write );
     if(!out)
