@@ -1,7 +1,8 @@
+
 // -*- Mode: c++; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4; -*-
 /* This file is part of the KDE project
    Copyright (C) 1998, 1999 Reginald Stadlbauer <reggie@kde.org>
-   Copyright (C) 2002-2004 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2002-2005 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -30,6 +31,7 @@
 #include <qpixmap.h>
 #include <qpointarray.h>
 #include <qvaluevector.h>
+#include <qtimer.h>
 
 #include <koRuler.h>
 #include <koQueryTrader.h>
@@ -37,6 +39,7 @@
 #include "global.h"
 #include <koRect.h>
 #include <kotextformat.h>
+#include "presstep.h"
 
 class KURL;
 class KoTextFormatInterface;
@@ -46,7 +49,7 @@ class QPainter;
 class KTextEdit;
 class KPTextObject;
 class KPObject;
-class QPopupMenu;
+class KPopupMenu;
 class QResizeEvent;
 class QPaintEvent;
 class QDragEnterEvent;
@@ -69,6 +72,8 @@ class KPrinter;
 class KPTextView;
 class KPPartObject;
 class KCommand;
+class EffectHandler;
+class KPPageEffects;
 /**
  * Class KPCanvas - There is a single instance of this class for a given view.
  *
@@ -80,8 +85,9 @@ class KPrCanvas : public QWidget
 
 public:
 
-    // constructor - destructor
+    /// constructor
     KPrCanvas( QWidget *parent=0,const char *name=0,KPresenterView *_view=0 );
+    /// destructor
     ~KPrCanvas();
 
     KPresenterView * getView()const { return m_view; }
@@ -126,19 +132,33 @@ public:
     void setDecreaseFontSize();
     KPTextObject* kpTxtObj() const;
 
-    // Start a screen presentation
-    // All pages if curPgNum is -1, otherwise just curPgNum (1-based)
-    void startScreenPresentation( float presFakt, int curPgNum = -1);
+    /**
+     * Start a screen presentation
+     * All pages if @p curPgNum is -1, otherwise just @p curPgNum (1-based)
+     */
+    void startScreenPresentation( double zoomX, double zoomY, int curPgNum = -1);
     void stopScreenPresentation();
-    bool pNext( bool );
-    bool pPrev( bool );
-    void setNextPageTimer( bool _nextPageTimer ) { nextPageTimer = _nextPageTimer; }
 
-    unsigned int presPage() const { return currPresPage; } // 1 based
-    int presStep() const { return currPresStep; }
-    float presFakt() const { return _presFakt; }
-    int numPresSteps() const { return presStepList.count(); }
-    int numPresPages() const { return slideList.count(); }
+    /**
+     * Go to the next step in the presentation.
+     * gotoNextPage: if is set to true go to beginning of the
+     *               next page.
+     */
+    bool pNext( bool gotoNextPage = false );
+
+    /**
+     * Go to previous step of the presentation.
+     * gotoPreviousPage: if is set to true go to beginning of the
+     *                   previous page.
+     */
+    bool pPrev( bool gotoPreviousPage = false );
+
+    /// returns the current page of the presentation 1 based
+    unsigned int presPage() const { return m_step.m_pageNumber + 1; }
+    /// returns the current step of the presentation
+    int presStep() const { return m_step.m_step; }
+    int numPresSteps() const { return m_pageEffectSteps.count(); }
+    int numPresPages() const { return m_presentationSlides.count(); }
 
     bool canAssignEffect( QPtrList<KPObject> &objs ) const;
 
@@ -170,29 +190,29 @@ public:
                         int forceHeight = 0 );
 
     /**
-    \brief Save page to bitmap file.
+     \brief Save page to bitmap file.
 
-    Export a page of the currently open presentation to disk
-    using a bitmap format like e.g. PNG.
-    This method uses a QPixmap::save() call.
+     Export a page of the currently open presentation to disk
+     using a bitmap format like e.g. PNG.
+     This method uses a QPixmap::save() call.
 
-    \param nPage the internally used <b>0-based</b> page number
-    \param nWidth the desired image width in px
-    \param nHeight the desired image height in px
-    \param fileURL the URL of the image file to be created: if this
+     \param nPage the internally used <b>0-based</b> page number
+     \param nWidth the desired image width in px
+     \param nHeight the desired image height in px
+     \param fileURL the URL of the image file to be created: if this
        does not point to a local file a KTempFile is created by QPixmap::save()
        which is then copied over to the desired location.
-    \param format the format of the image file (see QPixmap::save())
-    \param quality the quality of the image (see QPixmap::save())
+     \param format the format of the image file (see QPixmap::save())
+      \param quality the quality of the image (see QPixmap::save())
 
-    example:
+     example:
 \verbatim
 exportPage( 0, s, 800, 600, "/home/khz/page0.png", "PNG", 100 );
 \endverbatim
-    \returns True if the file was written successfully.
+     \returns True if the file was written successfully.
 
-    \sa drawPageInPix, KPresenterViewIface::exportPage
-    */
+     \sa drawPageInPix, KPresenterViewIface::exportPage
+     */
     bool exportPage( int nPage, int nWidth, int nHeight,
                      const KURL& fileURL,
                      const char* format,
@@ -206,7 +226,6 @@ exportPage( 0, s, 800, 600, "/home/khz/page0.png", "PNG", 100 );
      */
     void presGotoFirstPage();
 
-
     KPrPage* activePage() const;
 
     bool oneObjectTextExist() const;
@@ -238,9 +257,10 @@ exportPage( 0, s, 800, 600, "/home/khz/page0.png", "PNG", 100 );
     // get - set data
     const QPtrList<KPObject> & getObjectList() const;
 
+    void playSound( const QString &soundFileName );
     void stopSound();
 
-    //for KPTextView
+    ///for KPTextView
     void dragStarted() { mousePressed = false; }
 
     void setXimPosition( int x, int y, int w, int h, QFont *f );
@@ -256,13 +276,12 @@ exportPage( 0, s, 800, 600, "/home/khz/page0.png", "PNG", 100 );
     void insertPie( const QRect& );
     KPTextObject* insertTextObject( const QRect& );
     void insertPicture( const QRect &_r, const QPoint & tl = QPoint());
-    void changePicture( const KURL & url );
+    void changePicture( const KURL & url, QWidget *window );
 
     int numberOfObjectSelected() const;
     KPObject *getSelectedObj() const;
     KPPixmapObject *getSelectedImage() const;
 
-    int getPenBrushFlags() const;
     bool getEditMode() const { return editMode; }
 
     void ungroupObjects();
@@ -302,61 +321,23 @@ exportPage( 0, s, 800, 600, "/home/khz/page0.png", "PNG", 100 );
 
     void flipObject( bool _horizontal );
 
+    void lowerObjs( bool backward );
+    void raiseObjs( bool forward );
+
     KCommand *setKeepRatioObj( bool p );
     KCommand *setProtectSizeObj(bool protect);
 
     KoRect zoomAllObject();
 
-
-    //return pos of kptextobj
-    int textObjectNum( KPTextObject * obj ) const;
-    KPTextObject * textObjectByPos( int pos ) const;
-
-    void alignObjLeft();
-    void alignObjCenterH();
-    void alignObjRight();
-    void alignObjTop();
-    void alignObjCenterV();
-    void alignObjBottom();
+    /**
+     * Align all selected and not protected objects on the
+     * active and on the sticky page according to the AlignType
+     */
+    void alignObjects( AlignType at );
 
     bool canMoveOneObject()const;
     QPtrList<KPTextObject> listOfTextObjs() const;
 
-    bool getSticky( bool sticky ) const;
-
-    bool getProtect( bool prot ) const;
-    bool getKeepRatio( bool _ratio ) const;
-    //return true if we are selected several object and there are not
-    //the same "keep ratio" properties"
-    bool differentKeepRatio( bool p )const;
-    bool differentProtect( bool p )const;
-
-    bool getProtectContent(bool prot) const;
-    QPen getPen( const QPen & _pen )const;
-    QBrush getBrush( const QBrush & _brush )const;
-    LineEnd getLineBegin( LineEnd  _end )const;
-    LineEnd getLineEnd( LineEnd  _end )const;
-    FillType getFillType( FillType _fillType ) const;
-    QColor getGColor1( const QColor & _col )const;
-    QColor getGColor2( const QColor & _col )const;
-    BCType getGType( BCType _gt )const;
-    bool getGUnbalanced( bool _g )const;
-    int getGXFactor( int _g )const;
-    int getGYFactor( int _g )const;
-    int getRndY( int _ry )const;
-    int getRndX( int _rx )const;
-    int getPieAngle( int pieAngle )const;
-    int getPieLength( int pieLength )const;
-    PieType getPieType( PieType pieType )const;
-    bool getCheckConcavePolygon( bool check ) const;
-    int getCornersValue( int corners ) const;
-    int getSharpnessValue( int sharpness ) const;
-    PictureMirrorType getPictureMirrorType( PictureMirrorType type ) const;
-    int getPictureDepth( int depth ) const;
-    bool getPictureSwapRGB( bool swapRGB ) const;
-    bool getPictureGrayscal( bool grayscal ) const;
-    int getPictureBright( int bright ) const;
-    ImageEffect getImageEffect(ImageEffect eff) const;
     QPixmap getPicturePixmap() const;
     KCommand *setProtectContent( bool b );
     void closeObject(bool _close);
@@ -374,6 +355,7 @@ public slots:
     void clipPaste();
     void deleteObjs();
     void copyObjs();
+    void copyOasisObjs();
 
     void rotateObjs();
     void shadowObjs();
@@ -394,18 +376,25 @@ signals:
     void fontChanged( const QFont & );
     void colorChanged( const QColor & );
     void alignChanged( int );
-    void updateSideBarItem( int );
-    void stopPres();
+    /**
+     * Emmitted when presentation should be restarted
+     * This is used in automatic screen and infinite loop mode.
+     */
+    void stopAutomaticPresentation();
+    /**
+     * Emmitted when presentation should be restarted
+     * This is used in automatic screen and infinite loop mode.
+     */
+    void restartPresentation();
     void objectSelectedChanged();
     void objectSizeChanged();
-    // Emitted when the current frameset edit changes
+    /// Emitted when the current frameset edit changes
     void currentObjectEditChanged();
 
     void selectionChanged( bool hasSelection );
     void sigMouseWheelEvent( QWheelEvent * );
 
 protected:
-
     struct PicCache
     {
         QPicture pic;
@@ -414,27 +403,75 @@ protected:
     };
 
     // functions for displaying
+    /**
+     * This method is used for repainting the canvas.
+     */
     virtual void paintEvent( QPaintEvent* );
-    void drawBackground( QPainter*, const QRect& ) const;
 
-    //draw all object : object in current page + object in sticky page
-    void drawObjects( QPainter*, const QRect&, bool drawCursor,
-                      SelectionMode selectionMode, bool doSpecificEffects ) const;
+    /**
+     * Draws the background of page to painter.
+     * Set edit to true if in edit mode, false in presentation mode.
+     */
+    void drawBackground( QPainter* painter, const QRect& rect, KPrPage * page, bool edit = false ) const;
 
-    void drawObjectsInPage(QPainter *painter, const KoRect& rect2, bool drawCursor,
-                           SelectionMode selectionMode, bool doSpecificEffects,
-                           const QPtrList<KPObject> & obj) const;
+    /**
+     * Draw obj to painter.
+     */
+    void drawAllObjectsInPage( QPainter *painter, const QPtrList<KPObject> & obj, int pageNum ) const;
 
-    void drawAllObjectsInPage( QPainter *painter, const QPtrList<KPObject> & obj ) const;
+    /**
+     * Draw _objects shown at step to painter.
+     * This method is used for presentation mode, printing.
+     */
+    void drawObjectsPres( QPainter *painter, const QPtrList<KPObject> &_objects, PresStep step ) const;
 
-    // draw all helpline
+    /**
+     * Draw _objects to painter.
+     * Only rect is painted.
+     * This method is used for edit mode.
+     */
+    void drawObjectsEdit( QPainter *painter, const KoRect &rect, const QPtrList<KPObject> &_objects,
+                          SelectionMode selectionMode, int page ) const;
+
+    /**
+     * Draw _objects to painter.
+     * contour if true only countor of selected objects is drawn
+     * selectionMode selected objects use this selection mode
+     * textView if set print editable textobject (used by drawObjectsEdit)
+     * This method is used by drawObjectsPres and drawObjectsEdit.
+     */
+    void drawObjects( QPainter *painter, const QPtrList<KPObject> &objects, SelectionMode selectionMode,
+                      bool contour, KPTextView * textView, int pageNum ) const;
+
+    /**
+     * Draw _objects of page to painter.
+     * This method is used for edit mode.
+     */
+    void drawEditPage( QPainter *painter, const QRect &_rect,
+                       KPrPage *page, SelectionMode selectionMode ) const;
+
+    /**
+     * Draw _objects of page shown at step to painter.
+     * This method is used for presentation mode, printing.
+     */
+    void drawPresPage( QPainter *painter, const QRect &_rect, PresStep step ) const;
+
+    /// draw all helpline
     void drawHelplines(QPainter *painter, const QRect &rect2) const;
 
     void drawHelpPoints( QPainter *painter, const QRect &rect2) const;
 
-    // draw grid
+    /// draw grid
     void drawGrid(QPainter *painter, const QRect &rect2) const;
 
+    /**
+     * Finish the object effects.
+     * This shown the last step of the effect. It stops the effect timer and
+     * disconnect it and the effect handler deleted.
+     */
+    bool finishObjectEffects();
+
+    bool finishPageEffect( bool cancel = false );
 
     QRect getOldBoundingRect( const KPObject *obj );
 
@@ -445,7 +482,13 @@ protected:
     virtual void mouseDoubleClickEvent( QMouseEvent *e );
     virtual void wheelEvent( QWheelEvent *e );
     virtual void resizeEvent( QResizeEvent *e );
-    KPObject *getObjectAt( const KoPoint&pos );
+    /**
+     * Returns the object at position pos
+     * @param pos the position in the document on which to look
+     * @param withoutProtected if set to true only unprotecred objects are returned
+     * @return the object at position pos
+     */
+    KPObject *getObjectAt( const KoPoint &pos, bool withoutProtected = false );
     virtual void focusInEvent( QFocusEvent* ) {}
     virtual void focusOutEvent( QFocusEvent* ) {}
     virtual void enterEvent( QEvent *e );
@@ -455,10 +498,13 @@ protected:
     virtual void dropEvent( QDropEvent *e );
     virtual void keyPressEvent( QKeyEvent *e );
     virtual void keyReleaseEvent( QKeyEvent *e );
+    virtual void imStartEvent( QIMEvent * e );
+    virtual void imComposeEvent( QIMEvent * e );
+    virtual void imEndEvent( QIMEvent * e );
 
     void eraseEmptySpace( QPainter * painter, const QRegion & emptySpaceRegion, const QBrush & brush ) const;
 
-    // setup popupmenus
+    /// setup popupmenus
     void setupMenus();
 
     unsigned int currPgNum() const;
@@ -467,17 +513,22 @@ protected:
     bool spManualSwitch() const;
     QRect getPageRect( bool decBorders = true ) const;
     unsigned int pageNums() const;
-    float objSpeedFakt() const;
-    float pageSpeedFakt() const;
     void calcRatio( double &dx, double &dy, ModifyType _modType, double ratio ) const;
 
     void _repaint( bool erase=true );
     void _repaint( const QRect &r );
     void _repaint( KPObject *o );
 
-    void printPage( QPainter*, int pageNum );
-    void doObjEffects();
-    void drawObject( KPObject*, QPixmap*, int, int, int, int, int, int );
+    /**
+     * Draw page defined in step to painter.
+     * This method is used for printing a presentation.
+     */
+    void printPage( QPainter*, PresStep step );
+
+    /**
+     * This method animates the objects in the presentation.
+     */
+    void doObjEffects( bool isAllreadyPainted = false );
 
     KPPartObject *insertObject( const QRect& );
     void insertAutoform( const QRect&, bool );
@@ -507,8 +558,6 @@ protected:
     void printRTDebug( int info );
 #endif
 
-    KPrPage *stickyPage() const;
-
     void moveHelpLine(const QPoint & pos);
     void tmpMoveHelpLine( const QPoint & newPos);
 
@@ -518,20 +567,27 @@ protected:
     KoRect getAlignBoundingRect() const;
 
 private:
-    QValueList<int> pages(const QString &range) const;
-    bool pagesHelper(const QString &chunk, QValueList<int> &list) const;
     void picViewOrigHelper(int x, int y);
 
     void moveObject( int x, int y, bool key );
+
+    //---- stuff needed for resizing ----
+    /// resize the m_resizeObject
     void resizeObject( ModifyType _modType, int _dx, int _dy );
+    /// create ResizeCmd
+    void finishResizeObject( const QString &name, int mx, int my, bool layout = true );
 
     void raiseObject( KPObject *_kpobject );
     void lowerObject();
     int selectedObjectPosition;
 
-    bool nextPageTimer;
-
-    void playSound( const QString &soundFileName );
+    /**
+     * This is used in automatic presentation mode.
+     * If it is set to true and we are in automatic presentation
+     * mode the timer of the page will be activated after the last
+     * effect has been shown.
+     */
+    bool m_setPageTimer;
 
     void drawPolygon( const KoPoint &startPoint, const KoPoint &endPoint );
 
@@ -541,6 +597,10 @@ private:
     QPoint limitOfPoint(const QPoint& _point) const;
 
     void calcBoundingRect();
+
+    /**
+     * Return true if object is a header/footer which is hidden.
+     */
     bool objectIsAHeaderFooterHidden(KPObject *obj) const;
 
     KPTextObject* textUnderMouse( const QPoint & point );
@@ -558,34 +618,64 @@ private slots:
     void toFontChanged( const QFont &font ) { emit fontChanged( font ); }
     void toColorChanged( const QColor &color ) { emit colorChanged( color ); }
     void toAlignChanged( int a ) { emit alignChanged( a ); }
-    void drawingMode();
-    void switchingMode();
+    /**
+     * Switch to drawing mode. In drawing mode it is possible to draw on the
+     * current slide.
+     * Only to be used in presentation mode.
+     */
+    void setDrawingMode();
+    /**
+     * Set switching mode. This ends drawing mode and goes back to
+     * stepping between the pages.
+     * If continueTimer is true the autoPres timer is continued
+     * Only to be used in presentation mode.
+     */
+    void setSwitchingMode( bool continueTimer = true );
     void slotGotoPage();
     void slotExitPres();
     void terminateEditing( KPTextObject * );
 
+    /**
+     * Do the next step of the object effect.
+     * This restarts the effect tmer. If the effects are
+     * completed the timer is disconnected and the effect handler
+     * deleted.
+     */
+    void slotDoEffect();
+
+    void slotDoPageEffect();
+
 private:
     // variables
-    QPopupMenu *presMenu;
+    /// Popup menu used in presentation mode.
+    KPopupMenu *m_presMenu;
     bool showingLastSlide;
     bool mousePressed;
     bool drawContour;
-    double startAngle;
     ModifyType modType;
     /**
      * Saves the last mouse position during mouse move events.
      */
     QPoint m_savedMousePos;
 
-    KPObject *resizeObjNum, *editNum, *rotateNum;
+    KPObject *editNum;
 
     bool fillBlack;
     KPresenterView *m_view;
-    bool editMode, goingBack, drawMode;
-    bool drawLineInDrawMode;
-    
+    bool editMode, goingBack;
     /**
-     * Save the lines drawed in drawMode. 
+     * True when we are in drawing mode.
+     * False when we are in switching mode.
+     */
+    bool m_drawMode;
+    /**
+     * True if the a line should be drawn.
+     * Used in drawing mode.
+     */
+    bool m_drawLineInDrawMode;
+
+    /**
+     * Save the lines drawed in drawMode.
      * This is used for paint events.
      * Used in drawing mode.
      */
@@ -596,30 +686,57 @@ private:
      * Used in drawing mode.
      */
     int m_drawModeLineIndex;
-    
+
     bool mouseSelectedObject;
-    unsigned int currPresPage, currPresStep, subPresStep;
-    unsigned int oldPresPage, oldPresStep, oldSubPresStep;
+    /// information about current step of the presentation
+    PresStep m_step;
     float _presFakt;
     int m_showOnlyPage; // 1-based (-1 = all)
-    QValueList<int> presStepList, slideList;
-    QValueList<int>::Iterator slideListIterator;
-    int PM_DM, PM_SM;
+    /// list of all effect steps occuring on the active page
+    QValueList<int> m_pageEffectSteps;
+    /// List of the slides used in the presentation
+    QValueList<int> m_presentationSlides;
+    /// Iterator over the slides of a presentation
+    QValueList<int>::Iterator m_presentationSlidesIterator;
+    /// EffectHandler for object effects
+    EffectHandler *m_effectHandler;
+
+    KPPageEffects *m_pageEffect;
+
+    /// EffectTimer
+    QTimer m_effectTimer;
+
+    QTimer m_pageEffectTimer;
+
+    /// menu identifier for draw mode
+    int PM_DM;
     int firstX, firstY;
-    double axisX, axisY;
     int delPageId;
     bool drawRubber;
-    QRect rubber, oldBoundingRect;
-    KoRect resizeRect;
+    QRect rubber;
+
+    //---- stuff needed for resizing ----
+    /// object which gets resized
+    KPObject *m_resizeObject;
+    /// size of the object at when resizing is started
+    KoRect m_rectBeforeResize;
+    /// should the ratio of the object kept during resize
+    bool m_keepRatio;
+    /// ratio of the object ( width / height )
+    double m_ratio;
+
+    //---- stuff needed for rotating ----
+    /// object which gets rotated
+    KPObject *m_rotateObject;
+    /// center of the rotated object
+    KoPoint m_rotateCenter;
+    /// angle of the object at start of rotate
+    double m_angleBeforeRotate;
+
     ToolEditMode toolEditMode;
     QRect insRect;
     KoDocumentEntry partEntry;
-    // List of objects to draw, from the previous step in the presentation
-    // (those that remain on screen between two steps)
-    QPtrList <KPObject> tmpObjs;
     QString autoform;
-    bool inEffect, keepRatio;
-    double ratio;
     QPixmap buffer;
 
     KPTextView *m_currentTextObjectView;
@@ -639,11 +756,11 @@ private:
     int m_xOffset, m_yOffset;
     int m_xOffsetSaved, m_yOffsetSaved; // saved when going fullscreen
 
-    KoRect m_boundingRect; // when moving object(s)
-    KoPoint m_hotSpot; // when moving frame(s)
+    KoRect m_boundingRect; ///< when moving object(s)
+    KoPoint m_hotSpot; ///< when moving frame(s)
 
-    KoPoint moveStartPosMouse; // start position for move with mouse
-    KoPoint moveStartPosKey; // start position for move with key
+    KoPoint moveStartPosMouse; ///< start position for move with mouse
+    KoPoint moveStartPosKey; ///< start position for move with key
     int m_tmpHelpPoint;
     KoPoint tmpHelpPointPos;
 
@@ -657,8 +774,8 @@ private:
     bool m_keyPressEvent;
     bool m_drawSymetricObject;
 
-    KoRect m_origBRect;  // Start rect for move
-    QPoint m_origPos;  // Start point for move
+    KoRect m_origBRect;  ///< Start rect for move
+    QPoint m_origPos;  ///< Start point for move
 };
 
 #endif // __KPRCANVAS__
